@@ -1,19 +1,15 @@
 from decimal import Decimal, InvalidOperation
 from datetime import datetime, date, time
 
-from pony.utils import is_utf8, simple_decorator, throw
+from pony.utils import is_utf8, simple_decorator, throw, localbase
 from pony.converting import str2date, str2datetime
 from pony.orm.ormtypes import LongStr, LongUnicode
 
 class DBException(Exception):
-    def __init__(exc, *args, **kwargs):
-        exceptions = kwargs.pop('exceptions', [])
-        assert not kwargs
-        if not args and exceptions:
-            if len(exceptions) == 1: args = getattr(exceptions[0], 'args', ())
-            else: args = ('Multiple exceptions have occured',)
+    def __init__(exc, original_exc, *args):
+        args = args or getattr(original_exc, 'args', ())
         Exception.__init__(exc, *args)
-        exc.exceptions = exceptions
+        exc.original_exc = original_exc
 
 class RowNotFound(DBException): pass
 class MultipleRowsFound(DBException): pass
@@ -46,29 +42,41 @@ class     NotSupportedError(DatabaseError): pass
 def wrap_dbapi_exceptions(func, provider, *args, **kwargs):
     dbapi_module = provider.dbapi_module
     try: return func(provider, *args, **kwargs)
-    except dbapi_module.NotSupportedError, e: raise NotSupportedError(exceptions=[e])
-    except dbapi_module.ProgrammingError, e: raise ProgrammingError(exceptions=[e])
-    except dbapi_module.InternalError, e: raise InternalError(exceptions=[e])
-    except dbapi_module.IntegrityError, e: raise IntegrityError(exceptions=[e])
-    except dbapi_module.OperationalError, e: raise OperationalError(exceptions=[e])
-    except dbapi_module.DataError, e: raise DataError(exceptions=[e])
-    except dbapi_module.DatabaseError, e: raise DatabaseError(exceptions=[e])
+    except dbapi_module.NotSupportedError, e: raise NotSupportedError(e)
+    except dbapi_module.ProgrammingError, e: raise ProgrammingError(e)
+    except dbapi_module.InternalError, e: raise InternalError(e)
+    except dbapi_module.IntegrityError, e: raise IntegrityError(e)
+    except dbapi_module.OperationalError, e: raise OperationalError(e)
+    except dbapi_module.DataError, e: raise DataError(e)
+    except dbapi_module.DatabaseError, e: raise DatabaseError(e)
     except dbapi_module.InterfaceError, e:
         if e.args == (0, '') and getattr(dbapi_module, '__name__', None) == 'MySQLdb':
-            throw(InterfaceError, 'MySQL server misconfiguration', exceptions=[e])
-        raise InterfaceError(exceptions=[e])
-    except dbapi_module.Error, e: raise Error(exceptions=[e])
-    except dbapi_module.Warning, e: raise Warning(exceptions=[e])
+            throw(InterfaceError, e, 'MySQL server misconfiguration')
+        raise InterfaceError(e)
+    except dbapi_module.Error, e: raise Error(e)
+    except dbapi_module.Warning, e: raise Warning(e)
 
 class DBAPIProvider(object):
     paramstyle = 'qmark'
     quote_char = '"'
     max_params_count = 200
 
+    dbapi_module = None
     dbschema_cls = None
     translator_cls = None
     sqlbuilder_cls = None
 
+    def __init__(provider, *args, **kwargs):
+        pool_mockup = kwargs.pop('pony_pool_mockup', None)
+        if pool_mockup: provider.pool = pool_mockup
+        else: provider.pool = provider.get_pool(*args, **kwargs)
+        connection = provider.connect()
+        provider.inspect_connection(connection)
+        provider.release(connection)
+
+    def inspect_connection(provider, connection):
+        provider.table_if_not_exists_syntax = True
+
     def get_default_entity_table_name(provider, entity):
         return entity.__name__
 
@@ -149,22 +157,61 @@ class DBAPIProvider(object):
         throw(TypeError, 'No database converter found for type %s' % py_type)
 
     def get_converter_by_py_type(provider, py_type):
-        return provider._get_converter_type_by_py_type(py_type)()
+        converter_cls = provider._get_converter_type_by_py_type(py_type)
+        return converter_cls(py_type)
 
     def get_converter_by_attr(provider, attr):
-        return provider._get_converter_type_by_py_type(attr.py_type)(attr)
+        py_type = attr.py_type
+        converter_cls = provider._get_converter_type_by_py_type(py_type)
+        return converter_cls(py_type, attr)
+
+    def get_pool(provider, *args, **kwargs):
+        return Pool(provider.dbapi_module, *args, **kwargs)
+
+    def set_optimistic_mode(provider, connection):
+        pass
+
+    def set_pessimistic_mode(provider, connection):
+        pass
+
+    def start_optimistic_save(provider, connection):
+        pass
+
+
+class Pool(localbase):
+    def __init__(pool, dbapi_module, *args, **kwargs): # called separately in each thread
+        pool.dbapi_module = dbapi_module
+        pool.args = args
+        pool.kwargs = kwargs
+        pool.con = None
+    def connect(pool):
+        if pool.con is None:
+            pool.con = pool.dbapi_module.connect(*pool.args, **pool.kwargs)
+        return pool.con
+    def release(pool, con):
+        assert con is pool.con
+        try: con.rollback()
+        except:
+            pool.close(con)
+            raise
+    def drop(pool, con):
+        assert con is pool.con
+        pool.con = None
+        con.close()
 
 class Converter(object):
     def __deepcopy__(converter, memo):
         return converter  # Converter instances are "immutable"
-    def __init__(converter, attr=None):
+    def __init__(converter, py_type, attr=None):
+        converter.py_type = py_type
         converter.attr = attr
         if attr is None: return
         kwargs = attr.kwargs.copy()
         converter.init(kwargs)
         for option in kwargs: throw(TypeError, 'Unknown option %r' % option)
     def init(converter, kwargs):
-        pass
+        attr = converter.attr
+        if attr and attr.args: unexpected_args(attr, attr.args)
     def validate(converter, val):
         return val
     def py2sql(converter, val):
@@ -173,9 +220,6 @@ class Converter(object):
         return val
 
 class BoolConverter(Converter):
-    def init(converter, kwargs):
-        attr = converter.attr
-        if attr and attr.args: unexpected_args(attr, attr.args)
     def validate(converter, val):
         return bool(val)
     def sql2py(converter, val):
@@ -184,10 +228,10 @@ class BoolConverter(Converter):
         return "BOOLEAN"
 
 class BasestringConverter(Converter):
-    def __init__(converter, attr=None):
+    def __init__(converter, py_type, attr=None):
         converter.max_len = None
         converter.db_encoding = None
-        Converter.__init__(converter, attr)
+        Converter.__init__(converter, py_type, attr)
     def init(converter, kwargs):
         attr = converter.attr
         if not attr.args: max_len = None
@@ -221,9 +265,9 @@ class UnicodeConverter(BasestringConverter):
         return BasestringConverter.validate(converter, val)
 
 class StrConverter(BasestringConverter):
-    def __init__(converter, attr=None):
+    def __init__(converter, py_type, attr=None):
         converter.encoding = 'ascii'  # for the case when attr is None
-        BasestringConverter.__init__(converter, attr)
+        BasestringConverter.__init__(converter, py_type, attr)
         converter.utf8 = is_utf8(converter.encoding)
     def init(converter, kwargs):
         BasestringConverter.init(converter, kwargs)
@@ -242,8 +286,7 @@ class StrConverter(BasestringConverter):
 
 class IntConverter(Converter):
     def init(converter, kwargs):
-        attr = converter.attr
-        if attr and attr.args: unexpected_args(attr, attr.args)
+        Converter.init(converter, kwargs)
         min_val = kwargs.pop('min', None)
         if min_val is not None and not isinstance(min_val, (int, long)):
             throw(TypeError, "'min' argument for attribute %s must be int. Got: %r" % (attr, min_val))
@@ -275,8 +318,7 @@ class IntConverter(Converter):
 class RealConverter(Converter):
     default_tolerance = None
     def init(converter, kwargs):
-        attr = converter.attr
-        if attr and attr.args: unexpected_args(attr, attr.args)
+        Converter.init(converter, kwargs)
         min_val = kwargs.pop('min', None)
         if min_val is not None:
             try: min_val = float(min_val)
@@ -314,9 +356,9 @@ class RealConverter(Converter):
         return 'REAL'
 
 class DecimalConverter(Converter):
-    def __init__(converter, attr=None):
+    def __init__(converter, py_type, attr=None):
         converter.exp = None  # for the case when attr is None
-        Converter.__init__(converter, attr)
+        Converter.__init__(converter, py_type, attr)
     def init(converter, kwargs):
         attr = converter.attr
         args = attr.args
@@ -372,9 +414,6 @@ class DecimalConverter(Converter):
         return 'DECIMAL(%d, %d)' % (converter.precision, converter.scale)
 
 class BlobConverter(Converter):
-    def init(converter, kwargs):
-        attr = converter.attr
-        if attr and attr.args: unexpected_args(attr, attr.args)
     def validate(converter, val):
         if isinstance(val, buffer): return val
         if isinstance(val, str): return buffer(val)
@@ -386,9 +425,6 @@ class BlobConverter(Converter):
         return 'BLOB'
 
 class DateConverter(Converter):
-    def init(converter, kwargs):
-        attr = converter.attr
-        if attr and attr.args: unexpected_args(attr, attr.args)
     def validate(converter, val):
         if isinstance(val, datetime): return val.date()
         if isinstance(val, date): return val
@@ -402,9 +438,6 @@ class DateConverter(Converter):
         return 'DATE'
 
 class DatetimeConverter(Converter):
-    def init(converter, kwargs):
-        attr = converter.attr
-        if attr and attr.args: unexpected_args(attr, attr.args)
     def validate(converter, val):
         if isinstance(val, datetime): return val
         if isinstance(val, basestring): return str2datetime(val)
