@@ -7,8 +7,9 @@ from decimal import Decimal
 from datetime import datetime, date, time
 from time import strptime
 
-from pony.orm import dbschema, sqltranslation, sqlbuilding, dbapiprovider
-from pony.orm.dbapiprovider import DBAPIProvider, wrap_dbapi_exceptions
+from pony.orm import dbschema, sqltranslation, dbapiprovider
+from pony.orm.sqlbuilding import SQLBuilder, join
+from pony.orm.dbapiprovider import DBAPIProvider, Pool, wrap_dbapi_exceptions
 from pony.utils import localbase, datetime2timestamp, timestamp2datetime, simple_decorator, absolutize_path, throw
 
 class SQLiteForeignKey(dbschema.ForeignKey):
@@ -23,10 +24,8 @@ class SQLiteTranslator(sqltranslation.SQLTranslator):
     sqlite_version = sqlite.sqlite_version_info
     row_value_syntax = False
 
-class SQLiteBuilder(sqlbuilding.SQLBuilder):
+class SQLiteBuilder(SQLBuilder):
     dialect = 'SQLite'
-    def POW(builder, expr1, expr2):
-        return 'pow(', builder(expr1), ', ', builder(expr2), ')'
     def TODAY(builder):
         return "date('now', 'localtime')"
     def NOW(builder):
@@ -43,11 +42,21 @@ class SQLiteBuilder(sqlbuilding.SQLBuilder):
         return 'cast(substr(', builder(expr), ', 15, 2) as integer)'
     def SECOND(builder, expr):
         return 'cast(substr(', builder(expr), ', 18, 2) as integer)'
+    def MIN(builder, *args):
+        if len(args) == 0: assert False
+        elif len(args) == 1: fname = 'MIN'
+        else: fname = 'min'
+        return fname, '(',  join(', ', map(builder, args)), ')'
+    def MAX(builder, *args):
+        if len(args) == 0: assert False
+        elif len(args) == 1: fname = 'MAX'
+        else: fname = 'max'
+        return fname, '(',  join(', ', map(builder, args)), ')'
 
 class SQLiteStrConverter(dbapiprovider.StrConverter):
     def py2sql(converter, val):
         if converter.utf8: return val
-        return val.decode(converter.encoding)    
+        return val.decode(converter.encoding)
 
 class SQLiteDecimalConverter(dbapiprovider.DecimalConverter):
     def sql2py(converter, val):
@@ -61,15 +70,15 @@ class SQLiteDecimalConverter(dbapiprovider.DecimalConverter):
         exp = converter.exp
         if exp is not None: val = val.quantize(exp)
         return str(val)
-    
+
 class SQLiteDateConverter(dbapiprovider.DateConverter):
     def sql2py(converter, val):
-        try:       
+        try:
             time_tuple = strptime(val[:10], '%Y-%m-%d')
             return date(*time_tuple[:3])
         except: return val
     def py2sql(converter, val):
-        return val.strftime('%Y-%m-%d')    
+        return val.strftime('%Y-%m-%d')
 
 class SQLiteDatetimeConverter(dbapiprovider.DatetimeConverter):
     def sql2py(converter, val):
@@ -77,7 +86,7 @@ class SQLiteDatetimeConverter(dbapiprovider.DatetimeConverter):
         except: return val
     def py2sql(converter, val):
         return datetime2timestamp(val)
-    
+
 class SQLiteProvider(DBAPIProvider):
     dbapi_module = sqlite
     dbschema_cls = SQLiteSchema
@@ -96,7 +105,7 @@ class SQLiteProvider(DBAPIProvider):
         (date, SQLiteDateConverter)
     ]
 
-    def _get_pool(provider, filename, create_db=False):
+    def get_pool(provider, filename, create_db=False):
         if filename == ':memory:': return MemPool()
         else:
             # When relative filename is specified, it is considered
@@ -104,17 +113,18 @@ class SQLiteProvider(DBAPIProvider):
             # Database instance is created
 
             # the list of frames:
-            # 3 - user code: db = Database(...)
-            # 2 - cut_exception decorator
-            # 1 - pony.orm.Database.__init__()
-            # 0 - pony.dbproviders.sqlite._get_pool()
+            # 4 - user code: db = Database(...)
+            # 3 - cut_exception decorator
+            # 2 - pony.orm.Database.__init__()
+            # 1 - pony.dbapiprovider.DBAPIProvider.__init__()
+            # 0 - pony.dbproviders.sqlite.get_pool()
 
             filename = absolutize_path(filename, frame_depth=4)
-            return Pool(filename, create_db)
+            return SQLitePool(filename, create_db)
 
 provider_cls = SQLiteProvider
 
-class Pool(localbase):
+class SQLitePool(Pool):
     def __init__(pool, filename, create_db): # called separately in each thread
         pool.filename = filename
         pool.create_db = create_db
@@ -128,16 +138,6 @@ class Pool(localbase):
         pool.con = con = sqlite.connect(filename)
         _init_connection(con)
         return con
-    def release(pool, con):
-        assert con is pool.con
-        try: con.rollback()
-        except:
-            pool.close(con)
-            raise
-    def drop(pool, con):
-        assert con is pool.con
-        pool.con = None
-        con.close()
 
 mem_connect_lock = Lock()
 
@@ -166,10 +166,10 @@ def _text_factory(s):
 
 def _init_connection(con):
     con.text_factory = _text_factory
-    con.create_function("pow", 2, pow)
+    con.create_function('power', 2, pow)
 
 def unexpected_args(attr, args):
-    throw(TypeError, 
+    throw(TypeError,
         'Unexpected positional argument%s for attribute %s: %r'
         % ((args > 1 and 's' or ''), attr, ', '.join(map(repr, args))))
 
@@ -240,7 +240,7 @@ class MemoryCursorWrapper(object):
         wrapper.obj = cur
     def __iter__(wrapper):
         return wrapper
-    
+
 sqlite_cur_methods = '''execute executemany executescript fetchone fetchmany fetchall
                         next close setinputsize setoutputsize'''.split()
 
@@ -251,7 +251,7 @@ sqlite_cur_properties = 'rowcount lastrowid description arraysize'.split()
 
 for p in sqlite_cur_properties:
     setattr(MemoryCursorWrapper, p, make_wrapper_property(p))
-    
+
 class SqliteMemoryDbThread(Thread):
     def __init__(mem_thread):
         Thread.__init__(mem_thread, name="SqliteMemoryDbThread")
