@@ -29,12 +29,14 @@ class Param(object):
         return '%s(%r)' % (param.__class__.__name__, param.key)
 
 class Value(object):
-    __slots__ = 'value',
-    def __init__(self, value):
+    __slots__ = 'paramstyle', 'value'
+    def __init__(self, paramstyle, value):
+        self.paramstyle = paramstyle
         self.value = value
     def __unicode__(self):
         value = self.value
         if value is None: return 'null'
+        if isinstance(value, bool): return value and '1' or '0'
         if isinstance(value, (int, long, float, Decimal)): return str(value)
         if isinstance(value, basestring): return self.quote_str(value)
         if isinstance(value, datetime): return self.quote_str(datetime2timestamp(value))
@@ -44,6 +46,7 @@ class Value(object):
     def __repr__(self):
         return '%s(%r)' % (self.__class__.__name__, self.value)
     def quote_str(self, s):
+        if self.paramstyle in ('format', 'pyformat'): s = s.replace('%', '%%')
         return "'%s'" % s.replace("'", "''")
 
 def flat(tree):
@@ -66,7 +69,7 @@ def flat_conditions(conditions):
         if condition[0] == AND:
             result.extend(flat_conditions(condition[1:]))
         else: result.append(condition)
-    return result       
+    return result
 
 def join(delimiter, items):
     items = iter(items)
@@ -146,6 +149,7 @@ class SQLBuilder(object):
         builder.ast = ast
         builder.indent = 0
         builder.keys = {}
+        builder.inner_join_syntax = options.INNER_JOIN_SYNTAX
         builder.result = flat(builder(ast))
         builder.sql = u''.join(map(unicode, builder.result)).rstrip('\n')
         if paramstyle in ('qmark', 'format'):
@@ -163,7 +167,7 @@ class SQLBuilder(object):
         else: throw(NotImplementedError, paramstyle)
         builder.params = params
         builder.layout = tuple(param.key for param in params)
-        builder.adapter = adapter 
+        builder.adapter = adapter
     def __call__(builder, ast):
         if isinstance(ast, basestring):
             throw(AstError, 'An SQL AST list was expected. Got string: %r' % ast)
@@ -198,7 +202,7 @@ class SQLBuilder(object):
         return result
     def subquery(builder, *sections):
         builder.indent += 1
-        if not options.INNER_JOIN_SYNTAX:
+        if not builder.inner_join_syntax:
             sections = move_conditions_from_inner_join_to_where(sections)
         result = [ builder(s) for s in sections ]
         builder.indent -= 1
@@ -260,6 +264,9 @@ class SQLBuilder(object):
         return result
     def FROM(builder, *sources):
         return builder.sql_join('INNER', sources)
+    def INNER_JOIN(builder, *sources):
+        builder.inner_join_syntax = True
+        return builder.sql_join('INNER', sources)
     @indentable
     def LEFT_JOIN(builder, *sources):
         return builder.sql_join('LEFT', sources)
@@ -321,7 +328,7 @@ class SQLBuilder(object):
     def ROW(builder, *items):
         return '(', join(', ', map(builder, items)), ')'
     def VALUE(builder, value):
-        return [ builder.make_value(value) ]
+        return [ builder.make_value(builder.paramstyle, value) ]
     def AND(builder, *cond_list):
         cond_list = [ builder(condition) for condition in cond_list ]
         return join(' AND ', cond_list)
@@ -330,7 +337,9 @@ class SQLBuilder(object):
         return '(', join(' OR ', cond_list), ')'
     def NOT(builder, condition):
         return 'NOT (', builder(condition), ')'
-    
+    def POW(builder, expr1, expr2):
+        return 'power(', builder(expr1), ', ', builder(expr2), ')'
+
     EQ  = make_binary_op(' = ')
     NE  = make_binary_op(' <> ')
     LT  = make_binary_op(' < ')
@@ -341,9 +350,8 @@ class SQLBuilder(object):
     SUB = make_binary_op(' - ', True)
     MUL = make_binary_op(' * ', True)
     DIV = make_binary_op(' / ', True)
-    POW = make_binary_op(' ** ')
 
-    def CONCAT(builder, *args):        
+    def CONCAT(builder, *args):
         return '(',  join(' || ', map(builder, args)), ')'
     def NEG(builder, expr):
         return '-(', builder(expr), ')'
@@ -403,12 +411,12 @@ class SQLBuilder(object):
     def MIN(builder, *args):
         if len(args) == 0: assert False
         elif len(args) == 1: fname = 'MIN'
-        else: fname = 'min'
+        else: fname = 'least'
         return fname, '(',  join(', ', map(builder, args)), ')'
     def MAX(builder, *args):
         if len(args) == 0: assert False
         elif len(args) == 1: fname = 'MAX'
-        else: fname = 'max'
+        else: fname = 'greatest'
         return fname, '(',  join(', ', map(builder, args)), ')'
     def SUBSTR(builder, expr, start, len=None):
         if len is None: return 'substr(', builder(expr), ', ', builder(start), ')'
