@@ -90,7 +90,7 @@ class SQLTranslator(ASTTranslator):
             if obj.__class__.__dict__.get(func.__name__) is not func: throw(NotImplementedError)
             monad = translator.MethodMonad(translator, entity_monad, func.__name__)
         elif isinstance(node, ast.Name) and node.name in ('True', 'False'):
-            value = node.name == 'True' and 1 or 0
+            value = node.name == 'True' and True or False
             monad = translator.ConstMonad.new(translator, value)
         elif tt is tuple:
             monad = translator.ListMonad(translator, [ ParamMonad.new(translator, item_type, (src, i))
@@ -239,8 +239,8 @@ class SQLTranslator(ASTTranslator):
                 if isinstance(if_.monad, translator.AndMonad): cond_monads = if_.monad.operands
                 else: cond_monads = [ if_.monad ]
                 for m in cond_monads:
-                    if not m.aggregated: translator.conditions.append(m.getsql())
-                    else: translator.having_conditions.append(m.getsql())
+                    if not m.aggregated: translator.conditions.extend(m.getsql())
+                    else: translator.having_conditions.extend(m.getsql())
 
         translator.inside_expr = True
         translator.dispatch(tree.expr)
@@ -265,13 +265,12 @@ class SQLTranslator(ASTTranslator):
                 tableref = monad.make_tableref(translator.subquery)
             else: assert False
             translator.tableref = tableref
-            alias, pk_columns = tableref.make_join(pk_only=parent_translator is not None)
+            pk_only = parent_translator is not None or translator.aggregated
+            alias, pk_columns = tableref.make_join(pk_only=pk_only)
             translator.alias = alias
             translator.expr_type = entity
             translator.expr_columns = [ [ 'COLUMN', alias, column ] for column in pk_columns ]
             translator.row_layout = None
-            discr_criteria = entity._construct_discriminator_criteria_()
-            if discr_criteria: translator.conditions.insert(0, discr_criteria)
         else:
             translator.alias = None
             if isinstance(monad, translator.ListMonad):
@@ -531,6 +530,33 @@ class SQLTranslator(ASTTranslator):
         return expr_monad[lower:upper]
     def postSliceobj(translator, node):
         pass
+    def postIfExp(translator, node):
+        test_monad, then_monad, else_monad = node.test.monad, node.then.monad, node.else_.monad
+        if test_monad.type is not bool: test_monad = test_monad.nonzero()
+        result_type = coerce_types(then_monad.type, else_monad.type)
+        test_sql, then_sql, else_sql = test_monad.getsql()[0], then_monad.getsql(), else_monad.getsql()
+        if len(then_sql) == 1: then_sql, else_sql = then_sql[0], else_sql[0]
+        elif not translator.row_value_syntax: throw(NotImplementedError)
+        else: then_sql, else_sql = [ 'ROW' ] + then_sql, [ 'ROW' ] + else_sql
+        expr = [ 'CASE', None, [ [ test_sql, then_sql ] ], else_sql ]
+        result = translator.ExprMonad.new(translator, result_type, expr)
+        result.aggregated = test_monad.aggregated or then_monad.aggregated or else_monad.aggregated
+        return result
+
+def coerce_monads(m1, m2):
+    result_type = coerce_types(m1.type, m2.type)
+    if result_type in numeric_types and bool in (m1.type, m2.type) and result_type is not bool:
+        translator = m1.translator
+        if translator.dialect == 'PostgreSQL':
+            if m1.type is bool:
+                new_m1 = NumericExprMonad(translator, int, [ 'TO_INT', m1.getsql()[0] ])
+                new_m1.aggregated = m1.aggregated
+                m1 = new_m1
+            if m2.type is bool:
+                new_m2 = NumericExprMonad(translator, int, [ 'TO_INT', m2.getsql()[0] ])
+                new_m2.aggregated = m2.aggregated
+                m2 = new_m2
+    return result_type, m1, m2                
 
 max_alias_length = 30
 
@@ -578,13 +604,16 @@ class TableRef(object):
         tableref.entity = entity
         tableref.joined = False
     def make_join(tableref, pk_only=False):
+        entity = tableref.entity
         if not tableref.joined:
             subquery = tableref.subquery
-            subquery.from_ast.append([ tableref.alias, 'TABLE', tableref.entity._table_ ])
-            discr_criteria = tableref.entity._construct_discriminator_criteria_()
-            if discr_criteria: subquery.conditions.append(discr_criteria)
+            subquery.from_ast.append([ tableref.alias, 'TABLE', entity._table_ ])
+            if entity._discriminator_attr_:
+                discr_criteria = entity._construct_discriminator_criteria_(tableref.alias)
+                assert discr_criteria is not None
+                subquery.conditions.append(discr_criteria)
             tableref.joined = True
-        return tableref.alias, tableref.entity._pk_columns_
+        return tableref.alias, entity._pk_columns_
 
 class JoinedTableRef(object):
     def __init__(tableref, subquery, name_path, parent_tableref, attr):
@@ -598,20 +627,22 @@ class JoinedTableRef(object):
         assert isinstance(tableref.entity, EntityMeta)
         tableref.joined = False
     def make_join(tableref, pk_only=False):
+        entity = tableref.entity
+        pk_only = pk_only and not entity._discriminator_attr_
         if tableref.joined:
             if pk_only or not tableref.optimized:
                 return tableref.alias, tableref.pk_columns
+        subquery = tableref.subquery
         attr = tableref.attr
         parent_pk_only = attr.pk_offset is not None or attr.is_collection
         parent_alias, left_pk_columns = tableref.parent_tableref.make_join(parent_pk_only)
         left_entity = attr.entity
-        right_entity = attr.py_type
-        pk_columns = right_entity._pk_columns_
+        pk_columns = entity._pk_columns_
         if not attr.is_collection:
             if not attr.columns:
                 reverse = attr.reverse
                 assert reverse.columns and not reverse.is_collection
-                alias = tableref.subquery.get_short_alias(tableref.name_path, right_entity.__name__)
+                alias = subquery.get_short_alias(tableref.name_path, entity.__name__)
                 join_cond = join_tables(parent_alias, alias, left_pk_columns, reverse.columns)
             else:
                 if attr.pk_offset is not None:
@@ -624,21 +655,21 @@ class JoinedTableRef(object):
                     tableref.optimized = True
                     tableref.joined = True
                     return parent_alias, left_columns
-                alias = tableref.subquery.get_short_alias(tableref.name_path, right_entity.__name__)
+                alias = subquery.get_short_alias(tableref.name_path, entity.__name__)
                 join_cond = join_tables(parent_alias, alias, left_columns, pk_columns)
-            tableref.subquery.from_ast.append([ alias, 'TABLE', right_entity._table_, join_cond ])
+            subquery.from_ast.append([ alias, 'TABLE', entity._table_, join_cond ])
         elif not attr.reverse.is_collection:
-            alias = tableref.subquery.get_short_alias(tableref.name_path, right_entity.__name__)
+            alias = subquery.get_short_alias(tableref.name_path, entity.__name__)
             join_cond = join_tables(parent_alias, alias, left_pk_columns, attr.reverse.columns)
-            tableref.subquery.from_ast.append([ alias, 'TABLE', right_entity._table_, join_cond ])
+            subquery.from_ast.append([ alias, 'TABLE', entity._table_, join_cond ])
         else:
             right_m2m_columns = attr.symmetric and attr.reverse_columns or attr.columns
             if not tableref.joined:
                 m2m_table = attr.table
-                m2m_alias = tableref.subquery.get_short_alias(None, 't')
+                m2m_alias = subquery.get_short_alias(None, 't')
                 reverse_columns = attr.symmetric and attr.columns or attr.reverse.columns
                 m2m_join_cond = join_tables(parent_alias, m2m_alias, left_pk_columns, reverse_columns)
-                tableref.subquery.from_ast.append([ m2m_alias, 'TABLE', m2m_table, m2m_join_cond ])
+                subquery.from_ast.append([ m2m_alias, 'TABLE', m2m_table, m2m_join_cond ])
                 if pk_only:
                     tableref.alias = m2m_alias
                     tableref.pk_columns = right_m2m_columns
@@ -648,9 +679,13 @@ class JoinedTableRef(object):
             elif tableref.optimized:
                 assert not pk_only
                 m2m_alias = tableref.alias
-            alias = tableref.subquery.get_short_alias(tableref.name_path, right_entity.__name__)
+            alias = subquery.get_short_alias(tableref.name_path, entity.__name__)
             join_cond = join_tables(m2m_alias, alias, right_m2m_columns, pk_columns)
-            tableref.subquery.from_ast.append([ alias, 'TABLE', right_entity._table_, join_cond ])
+            subquery.from_ast.append([ alias, 'TABLE', entity._table_, join_cond ])
+        if entity._discriminator_attr_:
+            discr_criteria = entity._construct_discriminator_criteria_(alias)
+            assert discr_criteria is not None
+            subquery.conditions.insert(0, discr_criteria)
         tableref.alias = alias
         tableref.pk_columns = pk_columns
         tableref.optimized = False
@@ -707,6 +742,9 @@ class Monad(object):
             expr = [ 'CASE', None, [ [ expr, [ 'VALUE', 1 ] ] ], [ 'VALUE', None ] ]
             count_kind = 'ALL'
         elif len(expr) == 1: expr = expr[0]
+        elif translator.dialect == 'PostgreSQL':
+            row = [ 'ROW' ] + expr
+            expr = [ 'CASE', None, [ [ [ 'IS_NULL', row ], [ 'VALUE', None ] ] ], row ]
         elif translator.row_value_syntax == True and translator.dialect != 'Oracle':
             expr = ['ROW'] + expr
         elif translator.dialect == 'SQLite':
@@ -845,7 +883,7 @@ def make_numeric_binop(op, sqlop):
         if isinstance(monad2, (translator.AttrSetMonad, translator.NumericSetExprMonad)):
             return translator.NumericSetExprMonad(op, sqlop, monad, monad2)
         if monad2.type == 'METHOD': raise_forgot_parentheses(monad2)
-        result_type = coerce_types(monad.type, monad2.type)
+        result_type, monad, monad2 = coerce_monads(monad, monad2)
         if result_type is None:
             throw(TypeError, _binop_errmsg % (type2str(monad.type), type2str(monad2.type), op))
         left_sql = monad.getsql()[0]
@@ -996,7 +1034,9 @@ class StringMixin(MonadMixin):
     def nonzero(monad):
         sql = monad.getsql()[0]
         translator = monad.translator
-        return translator.BoolExprMonad(translator, [ 'GT', [ 'LENGTH', sql ], [ 'VALUE', 0 ]])
+        result = translator.BoolExprMonad(translator, [ 'GT', [ 'LENGTH', sql ], [ 'VALUE', 0 ]])
+        result.aggregated = monad.aggregated
+        return result
     def len(monad):
         sql = monad.getsql()[0]
         translator = monad.translator
@@ -1150,7 +1190,9 @@ class ParamMonad(Monad):
         elif type is buffer: cls = translator.BufferParamMonad
         elif isinstance(type, EntityMeta): cls = translator.ObjectParamMonad
         else: throw(NotImplementedError, type)
-        return cls(translator, type, src)
+        result = cls(translator, type, src)
+        result.aggregated = False
+        return result
     def __new__(cls, *args):
         if cls is ParamMonad: assert False, 'Abstract class'
         return Monad.__new__(cls)
@@ -1217,7 +1259,9 @@ class ConstMonad(Monad):
         elif value_type is NoneType: cls = translator.NoneMonad
         elif value_type is buffer: cls = translator.BufferConstMonad
         else: throw(NotImplementedError, value_type)
-        return cls(translator, value)
+        result = cls(translator, value)
+        result.aggregated = False
+        return result
     def __new__(cls, *args):
         if cls is ConstMonad: assert False, 'Abstract class'
         return Monad.__new__(cls)
@@ -1258,7 +1302,7 @@ class BoolExprMonad(BoolMonad):
         monad.type = bool
         monad.sql = sql
     def getsql(monad, subquery=None):
-        return monad.sql
+        return [ monad.sql ]
     def negate(monad):
         translator = monad.translator
         sql = monad.sql
@@ -1280,7 +1324,6 @@ cmp_negate.update((b, a) for a, b in cmp_negate.items())
 class CmpMonad(BoolMonad):
     def __init__(monad, op, left, right):
         translator = left.translator
-        check_comparable(left, right, op)
         if op == '<>': op = '!='
         if left.type is NoneType:
             assert right.type is not NoneType
@@ -1290,10 +1333,13 @@ class CmpMonad(BoolMonad):
             elif op == '!=': op = 'is not'
         elif op == 'is': op = '=='
         elif op == 'is not': op = '!='
+        check_comparable(left, right, op)
+        result_type, left, right = coerce_monads(left, right)
         BoolMonad.__init__(monad, translator)
         monad.op = op
         monad.left = left
         monad.right = right
+        monad.aggregated = left.aggregated or right.aggregated
     def negate(monad):
         return monad.translator.CmpMonad(cmp_negate[monad.op], monad.left, monad.right)
     def getsql(monad, subquery=None):
@@ -1301,18 +1347,18 @@ class CmpMonad(BoolMonad):
         sql = []
         left_sql = monad.left.getsql()
         if op == 'is':
-            return sqland([ [ 'IS_NULL', item ] for item in left_sql ])
+            return [ sqland([ [ 'IS_NULL', item ] for item in left_sql ]) ]
         if op == 'is not':
-            return sqland([ [ 'IS_NOT_NULL', item ] for item in left_sql ])
+            return [ sqland([ [ 'IS_NOT_NULL', item ] for item in left_sql ]) ]
         right_sql = monad.right.getsql()
         assert len(left_sql) == len(right_sql)
         if op in ('<', '<=', '>', '>='):
             assert len(left_sql) == len(right_sql) == 1
-            return [ cmp_ops[op], left_sql[0], right_sql[0] ]
+            return [ [ cmp_ops[op], left_sql[0], right_sql[0] ] ]
         if op == '==':
-            return sqland([ [ 'EQ', a, b ] for (a, b) in zip(left_sql, right_sql) ])
+            return [ sqland([ [ 'EQ', a, b ] for (a, b) in zip(left_sql, right_sql) ]) ]
         if op == '!=':
-            return sqlor([ [ 'NE', a, b ] for (a, b) in zip(left_sql, right_sql) ])
+            return [ sqlor([ [ 'NE', a, b ] for (a, b) in zip(left_sql, right_sql) ]) ]
         assert False
 
 class LogicalBinOpMonad(BoolMonad):
@@ -1329,7 +1375,12 @@ class LogicalBinOpMonad(BoolMonad):
         BoolMonad.__init__(monad, items[0].translator)
         monad.operands = items
     def getsql(monad, subquery=None):
-        return [ monad.binop ] + [ operand.getsql() for operand in monad.operands ]
+        result = [ monad.binop ]
+        for operand in monad.operands:
+            operand_sql = operand.getsql()
+            assert len(operand_sql) == 1
+            result.extend(operand_sql)
+        return [ result ]
 
 class AndMonad(LogicalBinOpMonad):
     binop = 'AND'
@@ -1345,7 +1396,7 @@ class NotMonad(BoolMonad):
     def negate(monad):
         return monad.operand
     def getsql(monad, subquery=None):
-        return [ 'NOT', monad.operand.getsql() ]
+        return [ [ 'NOT', monad.operand.getsql()[0] ] ]
 
 special_functions = SQLTranslator.special_functions = {}
 
@@ -1474,6 +1525,11 @@ def minmax(monad, sqlop, *args):
         t3 = coerce_types(t, t2)
         if t3 is None: throw(IncomparableTypesError, t, t2)
         t = t3
+    if t3 in numeric_types and translator.dialect == 'PostgreSQL':
+        args = list(args)
+        for i, arg in enumerate(args):
+            if arg.type is bool:
+                args[i] = NumericExprMonad(translator, int, [ 'TO_INT', arg.getsql() ])
     sql = [ sqlop ] + [ arg.getsql()[0] for arg in args ]
     return translator.ExprMonad.new(translator, t, sql)
 
@@ -1622,6 +1678,10 @@ class AttrSetMonad(SetMixin, Monad):
             extra_grouping = True
             if translator.hint_join: make_aggr = lambda expr_list: [ 'COUNT', 'ALL' ]
             else: make_aggr = lambda expr_list: [ 'COUNT', 'ALL', [ 'COUNT', 'ALL' ] ]
+        elif translator.dialect == 'PostgreSQL':
+            row = [ 'ROW' ] + expr_list
+            expr = [ 'CASE', None, [ [ [ 'IS_NULL', row ], [ 'VALUE', None ] ] ], row ]
+            make_aggr = lambda expr_list: [ 'COUNT', 'DISTINCT', expr ]
         elif translator.row_value_syntax:
             make_aggr = lambda expr_list: [ 'COUNT', 'DISTINCT' ] + expr_list
         elif translator.dialect == 'SQLite':
@@ -1827,7 +1887,7 @@ class NumericSetExprMonad(SetMixin, Monad):
         t2 = right.type
         if isinstance(t2, SetType): t2 = t2.item_type
         translator = left.translator
-        result_type = coerce_types(t1, t2)
+        result_type, left, right = coerce_monads(left, right)
         if result_type not in numeric_types:
             throw(TypeError, _binop_errmsg % (type2str(left.type), type2str(right.type), op))
         Monad.__init__(monad, translator, result_type)
