@@ -1,29 +1,33 @@
 import unittest
 from pony.orm.core import *
-from testutils import raises_exception
 
 db = Database('sqlite', ':memory:')
 
 class Person(db.Entity):
     name = Required(unicode)
     friends = Set('Person', reverse='friends')
-
 db.generate_mapping(create_tables=True)
 
+
 class TestSymmetric(unittest.TestCase):
     def setUp(self):
         rollback()
-        db.execute('delete from Person')
-        db.execute('delete from Person_friends')
-        db.insert('Person', id=1, name='A')
-        db.insert('Person', id=2, name='B')
-        db.insert('Person', id=3, name='C')
-        db.insert('Person', id=4, name='D')
-        db.insert('Person', id=5, name='E')
-        db.insert('Person_friends', person=1, person_2=2)
-        db.insert('Person_friends', person=2, person_2=1)
-        db.insert('Person_friends', person=1, person_2=3)
-        db.insert('Person_friends', person=3, person_2=1)
+        for i in range(1, 6):
+            try:
+                p = Person[i]
+                p.delete()
+            except ObjectNotFound: break
+        else:
+            commit()
+        db.insert('person', id=1, name='A')
+        db.insert('person', id=2, name='B')
+        db.insert('person', id=3, name='C')
+        db.insert('person', id=4, name='D')
+        db.insert('person', id=5, name='E')
+        db.insert('person_friends', person=1, person_2=2)
+        db.insert('person_friends', person=2, person_2=1)
+        db.insert('person_friends', person=1, person_2=3)
+        db.insert('person_friends', person=3, person_2=1)
         commit()
         rollback()
 
@@ -42,7 +46,7 @@ class TestSymmetric(unittest.TestCase):
         p4 = Person[4]
         p1.friends.add(p4)
         commit()
-        rows = db.select("* from Person_friends order by person, person_2")
+        rows = db.select("* from person_friends order by person, person_2")
         self.assertEqual(rows, [(1,2), (1,3), (1,4), (2,1), (3,1), (4,1)])
 
     def test2a(self):
@@ -65,12 +69,12 @@ class TestSymmetric(unittest.TestCase):
         p2 = Person[2]
         p1.friends.remove(p2)
         commit()
-        rows = db.select("* from Person_friends order by person, person_2")
+        rows = db.select("* from person_friends order by person, person_2")
         self.assertEqual(rows, [(1,3), (3,1)])
 
     def test3a(self):
-        db.execute('delete from Person_friends')
-        db.insert('Person_friends', person=1, person_2=2)
+        db.execute('delete from person_friends')
+        db.insert('person_friends', person=1, person_2=2)
         p1 = Person[1]
         p2 = Person[2]
         p2_friends = set(p2.friends)
@@ -81,8 +85,8 @@ class TestSymmetric(unittest.TestCase):
             self.assertEqual(e.args[0], "Phantom object Person[1] appeared in collection Person[2].friends")
         else: self.assert_(False)
     def test3b(self):
-        db.execute('delete from Person_friends')
-        db.insert('Person_friends', person=1, person_2=2)
+        db.execute('delete from person_friends')
+        db.insert('person_friends', person=1, person_2=2)
         p1 = Person[1]
         p2 = Person[2]
         p1_friends = set(p1.friends)
