@@ -4,7 +4,7 @@ import re, os, os.path, sys, time, datetime, types, linecache
 
 from itertools import imap, ifilter, count as _count
 from operator import itemgetter
-from inspect import isfunction
+from inspect import isfunction, ismethod
 from time import strptime
 from os import urandom
 from codecs import BOM_UTF8, BOM_LE, BOM_BE
@@ -22,6 +22,8 @@ else: from threading import local as localbase
 
 def copy_func_attrs(new_func, old_func, decorator_name=None):
     if new_func is not old_func:
+        if not isfunction(old_func) and not ismethod(old_func):
+            throw(TypeError, 'Decorated object must be function or method. Got: %r' % old_func)
         new_func.__name__ = old_func.__name__
         new_func.__doc__ = old_func.__doc__
         new_func.__module__ = old_func.__module__
@@ -37,9 +39,9 @@ def copy_func_attrs(new_func, old_func, decorator_name=None):
 
 def simple_decorator(old_dec):
     def new_dec(old_func):
-        def new_func(*args, **kwargs):
+        def pony_wrapper(*args, **kwargs):
             return old_dec(old_func, *args, **kwargs)
-        return copy_func_attrs(new_func, old_func, old_dec.__name__)
+        return copy_func_attrs(pony_wrapper, old_func, old_dec.__name__)
     return copy_func_attrs(new_dec, old_dec, 'simple_decorator')
 
 @simple_decorator
@@ -49,14 +51,14 @@ def decorator(old_dec, old_func):
 
 @simple_decorator
 def decorator_with_params(old_dec, *args, **kwargs):
-    if len(args) == 1 and isfunction(args[0]) and not kwargs:
+    if len(args) == 1 and not kwargs:
         old_func = args[0]
         new_func = old_dec(old_func)
         return copy_func_attrs(new_func, old_func, old_dec.__name__)
-    def you_should_never_see_this_decorator(old_func):
+    def parameterized_decorator(old_func):
         new_func = old_dec(old_func, *args, **kwargs)
         return copy_func_attrs(new_func, old_func, old_dec.__name__)
-    return you_should_never_see_this_decorator
+    return parameterized_decorator
 
 @decorator
 def cut_traceback(old_func):
@@ -117,7 +119,7 @@ def is_ident(string):
 _name_parts_re = re.compile(r'''
             [A-Z][A-Z0-9]+(?![a-z]) # ACRONYM
         |   [A-Z][a-z]*             # Capitalized or single capital
-        |   [a-z+                   # all-lowercase
+        |   [a-z]+                  # all-lowercase
         |   [0-9]+                  # numbers
         |   _+                      # underscores
         ''', re.VERBOSE)
@@ -164,9 +166,9 @@ else: _absolute_re = re.compile(r'^/')
 def is_absolute_path(filename):
     return bool(_absolute_re.match(filename))
 
-def absolutize_path(filename, frame_depth=2):
+def absolutize_path(filename, frame_depth):
     if is_absolute_path(filename): return filename
-    code_filename = sys._getframe(frame_depth).f_code.co_filename
+    code_filename = sys._getframe(frame_depth+1).f_code.co_filename
     if not is_absolute_path(code_filename):
         if code_filename.startswith('<') and code_filename.endswith('>'):
             if pony.MODE == 'INTERACTIVE': raise ValueError(
@@ -234,8 +236,7 @@ def current_timestamp():
 
 def datetime2timestamp(d):
     result = d.isoformat(' ')
-    if len(result) == 19: return result + '.000'
-    elif len(result) > 23: return result[:23]
+    if len(result) == 19: return result + '.000000'
     return result
 
 def timestamp2datetime(t):
@@ -418,13 +419,6 @@ def getlines2(filename, lineno, context=1):
     index = lineno - 1 - start
     return lines, index
 
-def reraise(exc_class, exceptions):
-    try:
-        cls, exc, tb = exceptions[0]
-        msg = '%s: %s' % (cls.__name__, " ".join(tostring(arg) for arg in exc.args))
-        raise exc_class, exc_class(msg, exceptions), tb
-    finally: del tb
-
 def count(*args, **kwargs):
     if kwargs: return _count(*args, **kwargs)
     if len(args) != 1: return _count(*args)
